#!/bin/bash

PATH=/sbin:/usr/sbin:$PATH

err_report() {
    echo "$0: Error on line $1"
}

trap 'err_report $LINENO' ERR

# Name of chain to use
IPT=IPTABLES-AUTOAPPLY

# Make it failsafe
set -Eeu -o pipefail

# ---------------------------------------------------------
# ipsets
ipset create black-src hash:net timeout 600 comment > /dev/null 2>&1 || true
ipset create tmp hash:net timeout 600 comment

ipset add tmp 83.97.73.0/24 timeout 0 comment "LFNET-1964, RU-Redbyte-LLC"

ipset swap tmp black-src
ipset destroy tmp


ipset create black-src-set list:set timeout 600 comment > /dev/null 2>&1 || true
ipset create tmp list:set timeout 600 comment
ipset add tmp black-src --exist timeout 0 # blacklistet source IP, *immer* blocken
ipset swap tmp black-src-set
ipset destroy tmp


# Allow all traffic within a set of networks
MYNETWORKS=iptables-autoapply-mynetworks
ipset flush $MYNETWORKS

ipset create $MYNETWORKS hash:net > /dev/null 2>&1 || true
#ipset add $MYNETWORKS 62.50.100.2/27
ipset add $MYNETWORKS 10.0.0.0/8
ipset add $MYNETWORKS 192.168.0.0/16
ipset add $MYNETWORKS 172.16.0.0/12
ipset add $MYNETWORKS 127.0.0.0/8


# Default is to keep the existing policy untouched.
#iptables --policy INPUT DROP
#iptables --policy FORWARD DROP


# -------------------------------------------------------
# LF.net trusted networks
ipset create LF hash:net comment > /dev/null 2>&1 || true
ipset create tmp hash:net comment > /dev/null 2>&1

ipset add tmp 212.9.190.0/24 --comment "LF.net Office"
ipset add tmp 212.9.190.18/32 --comment "zen.lf.net"
ipset add tmp 212.9.160.200/32 --comment "outcast.lf.net"
ipset add tmp 213.178.172.16/32 --comment "volz.eu"
ipset add tmp 212.9.183.0/25 --comment "LF.net VPN network"
ipset add tmp 212.9.190.49/32 --comment "hydra3.lf.net"
ipset add tmp 62.50.111.16/28 --comment "monitor.lf.net"
ipset add tmp 62.50.100.152/32 --comment "monitor.volz.eu"

ipset swap tmp LF
ipset destroy tmp


# ---------------------------------------------------------
# Kundenspezifische ipsets

function cidr_load {
  CIDR=$1

  # Der Block, der in iptables verwendet wird:
  ipset create cidr-$CIDR hash:net > /dev/null 2>&1 || true

  # Ein temporaeres Set zum Befuellen:
  ipset create tmp-cidr-$CIDR hash:net maxelem 200000 > /dev/null 2>&1 || true
  ipset flush tmp-cidr-$CIDR

  # Try several locations...
  FILE=/usr/share/iptables-autoapply/cidr-lists/$CIDR.list
  if [ ! -f $FILE ]; then
    FILE=/etc/network/iptables-autoapply/cidr-lists/$CIDR.list
  fi

  grep -v '^#' $FILE | while read a; do
    echo "add tmp-cidr-$CIDR $a"
  done | ipset restore

  # Aktiven Block mit dem temporaeren tauschen u. dann temporaeren entfernen:
  ipset swap tmp-cidr-$CIDR cidr-$CIDR
  ipset destroy tmp-cidr-$CIDR
}

ipset -n list cidr-asia > /dev/null 2>&1 || cidr_load asia
ipset -n list custom-ede-topf > /dev/null 2>&1 || cidr_load custom-ede-topf

# Set of sets
ipset create accid-400183-block list:set timeout 600 > /dev/null 2>&1 || true
ipset flush accid-400183-block
ipset add accid-400183-block --exist cidr-asia timeout 0

ipset create accid-400308-block list:set timeout 600 > /dev/null 2>&1 || true
ipset flush accid-400308-block
ipset add accid-400308-block --exist cidr-custom-ede-topf timeout 0


# ---------------------------------------------------------
#
# - Moeglichkeit des sehr fruehen SRC-basierten Blockings.
# - fuer ad hoc Blocks, z.B. SYN-Defender usw.
#
iptables -t mangle -F PREROUTING
iptables -t mangle -A PREROUTING -m set --match-set black-src-set src -j DROP -m comment --comment "Dynamic src block"


# Initialization ----------------------------------------------------------------
iptables -N $IPT-INPUT 2> /dev/null || true                  # New chain
iptables -N $IPT-FORWARD 2> /dev/null || true                # New chain
iptables -N $IPT 2> /dev/null || true                        # New chain

iptables -F $IPT-INPUT
iptables -A $IPT-INPUT -j $IPT
iptables -A $IPT-INPUT -j RETURN

iptables -F $IPT-FORWARD
iptables -A $IPT-FORWARD -j REJECT -m comment --comment "DEFAULT-DENY"
iptables -A $IPT-FORWARD -j $IPT
iptables -A $IPT-FORWARD -j RETURN

iptables -F $IPT

# FIXME, geht das eleganter?
while iptables -D INPUT -j $IPT-INPUT; do echo .; done > /dev/null 2>&1

# We want our chain as the first entry.
iptables -I INPUT -j $IPT-INPUT

iptables -N DOCKER-USER > /dev/null 2>&1 || true
#iptables -F DOCKER-USER
while iptables -D DOCKER-USER -j $IPT-FORWARD; do echo .; done > /dev/null 2>&1
iptables -I DOCKER-USER -j $IPT-FORWARD

# Allow all established traffic
iptables -A $IPT -m state --state ESTABLISHED,RELATED -j RETURN

# Allow SSH from known sources ---------------------------
iptables -A $IPT -m set --match-set LF src -m state --state NEW -p tcp --dport 22 -j RETURN

# Custom rules ================================================================

# Allow ICMP ---------------------------------------------
iptables -A $IPT -p icmp -j RETURN -m comment --comment "ICMP"

# START kundenspezifische Blocks -------------------------------
iptables -A $IPT -m set --match-set accid-400183-block src -p tcp -d 212.118.186.105 -j DROP -m comment --comment "eisenwagner/400183"

iptables -A $IPT -m set --match-set accid-400308-block src -p tcp -d 212.118.186.222 -j DROP -m comment --comment "topf/400308"


# END kundenspezifische Blocks -------------------------------

iptables -A $IPT -m set --match-set $MYNETWORKS src,dst -j RETURN -m comment --comment "Private traffic"

iptables -A $IPT -m state --state NEW -p tcp --dport 53 -j RETURN -m comment --comment "DNS"
iptables -A $IPT -m state --state NEW -p udp --dport 53 -j RETURN -m comment --comment "DNS"

# DEBUG: allow all ssh incoming
#iptables -I $IPT -p tcp --dport 22 -j RETURN

# Allow incoming DNS queries
#echo4 "-A INPUT -m state --state NEW -p tcp --dport 53 -j RETURN"
#echo4 "-A INPUT -m state --state NEW -p udp --dport 53 -j RETURN"

# Allow incoming SMTP
#echo4 "-A INPUT -m state --state NEW -p tcp --dport 25 -j RETURN"

# Typische Service-Ports f. Docker Services
iptables -A $IPT -m state --state NEW -p tcp --dport 80 -j RETURN
iptables -A $IPT -m state --state NEW -p tcp --dport 443 -j RETURN
#iptables -A $IPT -m state --state NEW -p tcp --dport 9200 -j RETURN
#iptables -A $IPT -m state --state NEW -p tcp --dport 8080 -j RETURN

# https://superuser.com/questions/1471265/open-iptables-whithin-docker-compose-network-and-one-external-server
#iptables -A $IPT -m conntrack -p tcp --ctorigdst 212.9.161.112/32 --ctorigdstport 8022 -j RETURN


# Allow all from LAN systems
#iptables -A $IPT -d 10.0.0.0/8 -j RETURN
iptables -A $IPT -d 127.0.0.0/8 -j RETURN


# # Allow Bacula TCP/9102 from backup.lf.net --------------
# S=""
# S="$S 213.178.160.3/32"
# S="$S 213.178.162.248/32"
# S="$S 212.9.160.62/32"
# S="$S 212.9.160.106/32"
# S="$S 212.9.160.135/32"
# S="$S 212.9.161.106/32"
# for i in $S
# do
#   iptables -A $IPT -s $i -m state --state NEW -p tcp --dport 9102 -j RETURN -m comment --comment "backup.lf.net"
# done


#iptables -A $IPT -j LOG

# Last rule must always be a DROP!
iptables -A $IPT -j DROP


# IPv6 =======================================================

ip6tables --policy INPUT DROP
ip6tables -F INPUT

ip6tables --policy FORWARD DROP
ip6tables -F FORWARD

# Wenn alles sauber durchgelaufen ist entfernen wir hier die Deny rule.
iptables -D $IPT-FORWARD -j REJECT -m comment --comment "DEFAULT-DENY" || true
#echo "succ"

# EOF
